<?php

namespace app\components\modules\payment;

use framework\request\Request;
use app\models\Trans;

/**
 * payment with paystar
 *
 * @author	Shayan Kiyani <info@paystar.ir>
 * @since		1.0
 * @package	payment module
 * @copyright   (c) 2014-2016 all rights reserved
 */
class paystar extends Payment
{
	/**
	 * request to paystar gateway
	 *
	 * @param integer $id, trans id (primary key)
	 * @param integer $au, trans authority code
	 * @param integer $price, trans price
	 * @param array $module, information of this module
	 * @param integer $product, product id
	 *
	 * @access public
	 * @return void
	 */
	public function request($id, $au, $price, $module, $product)
	{
		$trans = Trans::model()->getTransFactor($id);
		$description = "شناسه فاکتور : {$id}";
		$description .= "\nایمیل خریدار : {$trans->transEmail}";
		if(!empty($trans->transMobile))
			$description .= "\nتلفن خریدار : {$trans->transMobile}";
		
		$res = $this->paystarapi($module['pin']['value'], $price, $this->getCallbackUrl($au), $description);		
		if(empty($res) or $res == -900) 
			$this->setFlash('danger', $this->lang()->getIndex('paystar', 'noconect'));
		else if(strlen($res['au']) > 5)
		{
			$this->updateAu($id, $res['au']);
			$this->redirect("http://{$res['address']}/paying/{$res['au']}");
		}	
		else
			$this->setFlash('danger', $this->lang()->getIndex('paystar', 'error') . $res['au']);		
	}

	/**
	 * request to paystar for verify transaction
	 *
	 * @param integer $id, trans id (primary key)
	 * @param integer $au, trans authority code
	 * @param integer $price, trans price
	 * @param array $module, information of this module
	 * @param integer $product, product id
	 *
	 * @access public
	 * @return array|boolean
	 */
	public function verify($id, $au, $price, $module, $product)
	{
		if(!Request::isPost('transid'))
		{
			$this->setFlash('danger', $this->lang()->getIndex('paystar', 'inputNotValid'));
			return false;
		}
		
		$res = $this->paystarapi($module['pin']['value'], $price, Request::getPost('transid'), '', true);
		if(empty($res))	
			$this->setFlash('danger', $this->lang()->getIndex('paystar', 'nopay'));
		else if($res == -900) 
			$this->setFlash('danger', $this->lang()->getIndex('paystar', 'noconect'));		
		else if($res == 1)
			return array('au' => Request::getPost('transid'));
		else
			$this->setFlash('danger', $this->lang()->getIndex('paystar', 'error') . $res);			
	}

	/**
	 * module fields for install this
	 *
	 * @access public
	 * @return array
	 */
	public function fields()
	{
		return array(
			'pin' => array(
				'label' => $this->lang()->getIndex('paystar', 'pin'),
				'value' => '',
			)
		);
	}
	
	private function paystarapi($pin='', $amount=0, $tc='', $desc='Test CardStore', $verify=false)
	{
		require_once dirname(__FILE__) . '/paystar/index.php';		
		$res = false;
		try 
		{
			$m = !empty($verify) ? 'verify' : 'create';
			
			$f = array(
				'amount' => round($amount),
				'pin' => $pin,
				!empty($verify) ? 'transid' : 'callback' => $tc 
			);	
			
			if(empty($verify))
				$f = array_merge($f, array('bank'=>$ws_config['dbank'],'description'=>$desc)); 				

			$f_ = '';
			foreach($f as $k=>$v) 
				$f_ .= $k . '=' . urlencode($v) . '&'; 
				
			rtrim($f_, '&');

			$ch = curl_init();
			curl_setopt($ch,CURLOPT_URL, "http://{$ws_config['address']}/api/{$m}/");
			curl_setopt($ch,CURLOPT_POST, count($f));
			curl_setopt($ch,CURLOPT_POSTFIELDS, $f_);
			curl_setopt($ch,CURLOPT_RETURNTRANSFER, true);
			$res = curl_exec($ch);
			curl_close($ch);				
		}
		catch(Exception $e) 
		{
			return -900;
		} 

		$res = !empty($verify) ? $res : array('au'=>$res,'address'=>$ws_config['address']);
		return !empty($res) ? $res : false;
	}	
}